"""
Author: Taylor B. tayjaybabee@gmail.com
Date: 2024-11-23 13:07:52
LastEditors: Taylor B. tayjaybabee@gmail.com
LastEditTime: 2024-11-27 20:23:34
FilePath: inspy_logger/engine/adapters/__init__.py
Description: 这是默认设置,可以在设置》工具》File Description中进行配置
"""
from inspy_logger.engine.adapters.thread import ThreadAdapter
